@@ -50,8 +50,12 @@ class TqdmDeprecationWarning(Exception):
 
 # Create global parallelism locks to avoid racing issues with parallel bars
 # works only if fork available (Linux, MacOSX, but not on Windows)
-mp_lock = mp.Lock()  # multiprocessing lock
-th_lock = th.Lock()  # thread lock
+try:
+    mp_lock = mp.RLock()  # multiprocessing lock
+    th_lock = th.RLock()  # thread lock
+except OSError:  # pragma: no cover
+    mp_lock = None
+    th_lock = None
 
 
 class TqdmDefaultWriteLock(object):
@@ -63,7 +67,7 @@ class TqdmDefaultWriteLock(object):
     """
     def __init__(self):
         global mp_lock, th_lock
-        self.locks = [mp_lock, th_lock]
+        self.locks = [lk for lk in [mp_lock, th_lock] if lk is not None]
 
     def acquire(self):
         for lock in self.locks:
@@ -185,7 +189,7 @@ class tqdm(object):
         out  : str
             Number with Order of Magnitude SI unit postfix.
         """
-        for unit in ['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z']:
+        for unit in ['', 'k', 'M', 'G', 'T', 'P', 'E', 'Z']:
             if abs(num) < 999.95:
                 if abs(num) < 99.95:
                     if abs(num) < 9.995:
@@ -272,7 +276,7 @@ class tqdm(object):
             The iteration unit [default: 'it'].
         unit_scale  : bool or int or float, optional
             If 1 or True, the number of iterations will be printed with an
-            appropriate SI metric prefix (K = 10^3, M = 10^6, etc.)
+            appropriate SI metric prefix (k = 10^3, M = 10^6, etc.)
             [default: False]. If any other non-zero number, will scale
             `total` and `n`.
         rate  : float, optional
@@ -444,6 +448,8 @@ class tqdm(object):
         # Add to the list of instances
         if "_instances" not in cls.__dict__:
             cls._instances = WeakSet()
+        if "_lock" not in cls.__dict__:
+            cls._lock = TqdmDefaultWriteLock()
         with cls._lock:
             cls._instances.add(instance)
         # Create the monitoring thread
@@ -917,7 +923,7 @@ class tqdm(object):
         return id(self)
 
     def __iter__(self):
-        ''' Backward-compatibility to use: for x in tqdm(iterable) '''
+        """Backward-compatibility to use: for x in tqdm(iterable)"""
 
         # Inlining instance variables as locals (speed optimisation)
         iterable = self.iterable
@@ -1211,6 +1217,7 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
 
     def moveto(self, n):
         self.fp.write(_unicode('\n' * n + _term_move_up() * -n))
+        self.fp.flush()
 
     def clear(self, nolock=False):
         """
